/*
 * Assuming prototype is loaded the object 
 * RadioSherpa provides methods to access 
 * radio data.
 * In general each method that obtains data from online may 
 * be called synchronously or asynchronously by 
 * passing a function as the async argument.
 * in the synchronous case the result is returned. In the 
 * asynchronous case the function passed in invoked with the 
 * listed return value.
 * Usage:
 * Get List of available Genres:
 * RadioSherpa.getGenreList()
 * 
 * Get List of Cities
 * RadioSherpa.getCityList()
 *
 * Get List of Stations
 * ReadioSherpa.getStationList()
 *
 * Get StationInformation
 * RadioSherpa.getStationInformation()
 *
 *
 * Get Station data update only
 * RadioSherpa.getUpdateStationInformation()
 *
 */
function RadioSherpaCreateRequest(){
    var xmlhttp;

    /** Special IE only code ... */
    /*@cc_on
          @if (@_jscript_version >= 5)
              try
              {
                  xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
              }
              catch (e)
              {
                  try
                  {
                      xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
                  }
                  catch (E)
                  {
                      xmlhttp = false;
                  }
             }
          @else
             xmlhttp = false;
        @end @*/

    /** Every other browser on the planet */
    if (!xmlhttp && typeof XMLHttpRequest != 'undefined')
    {
        try
        {
            xmlhttp = new XMLHttpRequest();
        }
        catch (e)
        {
            xmlhttp = false;
        }
    }

    return xmlhttp;
}

var RadioSherpa={
    http_get:function(url,handler,onerror){
        var req=RadioSherpaCreateRequest();
        req.open('GET',url,handler?true:false);
        if(handler){
          req.onreadystatechange=function(){
            //DEBUG("HTTP STATE:" + req.readyState + "|Status=" + req.status);
            if(req.readyState==4) {
              handler(req);
              req.onreadystatechange=null;
              req=null;
            }
          };
        }
        req.setRequestHeader("API-Version",RadioSherpa.Config.version);
        return req;
    },
    http_post:function(url,handler,onerror){
        var req=RadioSherpaCreateRequest();
        req.open('POST',url,handler?true:false);
        if(handler){
          req.onreadystatechange=function(){
            //DEBUG("HTTP STATE:" + req.readyState + "|Status=" + req.status + "|Text=" + req.responseText);
            if(req.readyState==4) {
              handler(req);
              req.onreadystatechange=null;
              req=null;
            }
          };
        }
        req.setRequestHeader("API-Version",RadioSherpa.Config.version);
        return req;
    },
    /** 
   * Configuration Of connection
   * the filed debug may be substituted with a user defined
   * debug function for handling errors and such
   */
    Config: {
	version: "1.3",
        site: "http://api.radiosherpa.com/cgi-bin/api/Api.py/",
	debug: function(x){},
	exception: function(e,where,req){ RadioSherpa.Config.debug("Error:(" + where + ")" + e ); }
	
    },
    toQueryString: function(hash) { 
        //Prototype Type of is not so good in 1.4.0
        //Doesnt Handle Arrays properly
        var retstr="";
        hash.map(function(pair){
            if(typeof(pair.value) == 'object'){
                try{
                    pair.value.each(function(x){
                        retstr+=pair.key + "=" + encodeURI(x) + "&";
                    });
                }catch(ex){
                }
            }else{
                retstr+=pair.key + "=" + encodeURI(pair.value) + "&";
            }
        });
        return retstr;
    },

    exceptionFunc: function(req,e){
        RadioSherpa.Config.exception(e,"HTTP",req);
    },
    completionFunction:function(async) {
        return function(transport,json){
            if(200 == transport.status) {
                var resptxt= transport.responseText ;
                var where="PARSING";
                try{
                    var res=json;
                    if(res ==null) res=eval('(' + resptxt + ')');
                    var lst=res.results;
                    where="CALLBACK";
                    if(async!=null) async(lst);
                    async=null;
                    return lst;
                }catch(e){
                    RadioSherpa.Config.exception( e ,where);
                }
            }
            if(async!=null) async([]);
            async=null;
            return [];

        };
    },

    /**
             * Get a list of all the cities.
             * returns list of city names
             * @param [async] optional function to be invoked with result
             */
    getCityList: function(async) {
        var url = encodeURI(RadioSherpa.Config.site + "GetCityList?ResultFormat=json");
        var func=RadioSherpa.completionFunction(async);
        var req = RadioSherpa.http_get(url, async?func:null);
        req.send(null);
        if(async == null) return func(async)(req,null)
    },

    /**
             * Get a list of all the genres.
             * @param async optional function to be invoked with result
             * @returns javascript array/list of string genre names
             *
             */
    getGenreList: function(async) {
        var url = encodeURI(RadioSherpa.Config.site + "GetGenreList?ResultFormat=json");
        var compFunc=RadioSherpa.completionFunction(async);
        var req = RadioSherpa.http_get(url,async?compFunc:null);
        req.send(null);
        if(async == null) return compFunc(async)(req,null)
        compFunc=null;
    },
    /**
             * Get the list of stations.
             * @param {hash/object} filter an object constructor or prototype Hash or JS Object containing the 
             * filtering criteria.
             * @param async optional function to be invoked with the result
             * @returns A list of javascript objects containing id,name,frequency,band
             * @example
             * var stat=RadioSherpa.getStationList();
             * var item1=stations[0];
             * item1 === { 'id' : 3,
             * 'call_letters': 'WBCN',
             * 'frequency':'104.1',
             *  'band':'FM',
             *  'channel_count': 2 } 
             *
           * var stationInfo=RadioSherpa.getStationInformation([item1.id]);
           */
    getStationList: function(filter,async) {
        var params="";
        if(filter != undefined) {
            var hash=$H(filter);
            params=RadioSherpa.toQueryString(hash);
	    if(params.length > 0) params = "&" + params;
        }
        var url = encodeURI(RadioSherpa.Config.site + "GetStationList?ResultFormat=json&Debug=yes") + params;
        var compFunc = RadioSherpa.completionFunction(async);
        var req = RadioSherpa.http_get(url,async?compFunc:null)
        req.send(null);
        if(async == null) return compFunc(req,null)
        compFunc=null;
    },
    /**
         * Get the information for each station specified in the list stations.
         * @param {int[]} stationlist list of stationid's
         * @params {function} [async] An optional function to be invoked with results
         * @returns A list of javascript objects containing statation information
         * @example
         * Each element of the results list looks like:
         * 
         * { 'id' : 6 ,
         * 'call_letters' : 'WBOS' ,
         * 'frequency' : '92.9' ,
         * 'band' : 'FM' ,
         * 'city' : 'Boston (Brookline)' ,
         * 'state' : 'MA' ,
         * 'zipcode' : '02215' ,
         * 'is_digital' : 'True' ,
         * 'station_art_url' : '' ,
         * 'channels' : [
           * { 'itunesurl' : 'itms://phobos.apple.com/WebObjects/MZSearch.woa/wa/com.apple.jingle.search.DirectAction/advancedSearchResults?artistTerm=Gomez&songTerm=up+next%3A' ,
           * 'album' : 'None' ,
           * 'largeImageURL' : 'http://www.stage.radiosherpa.net:8888/static/stationImages/wbos_fm_ch1_logo_lg.jpg' ,
           * 'stationGenres' : ['rock','adult contemporary'] ,
           * 'songPlayTime' : '2006-07-27 16:28:08' ,
           * 'artist' : 'Gomez' ,
           * 'smallImageURL' : 'http://www.stage.radiosherpa.net:8888/static/stationImages/wbos_fm_ch1_logo_sm.jpg' ,
           * 'song' : 'up next:' ,
           * 'channel' : '1' ,
         * 'amazonUrl' : 'None' },
         * { 'itunesurl' : 'itms://phobos.apple.com/WebObjects/MZSearch.woa/wa/com.apple.jingle.search.DirectAction/advancedSearchResults?artistTerm=SPICE+GIRLS&songTerm=SPICE+UP+YOUR+LIFE' ,
         * 'album' : 'Spiceworld' ,
         * 'largeImageURL' : 'http://www.stage.radiosherpa.net:8888/static/stationImages/wbos_fm_ch2_logo_lg.jpg' ,
         * 'stationGenres' : ['rock','folk','live performance'] ,
         * 'playurl':'<URL TO a site where the stream can be found>'
         * 'songPlayTime' : '2006-07-27 16:28:08' ,
         * 'artist' : 'SPICE GIRLS' ,
         * 'smallImageURL' : 'http://www.stage.radiosherpa.net:8888/static/stationImages/wbos_fm_ch2_logo_sm.jpg' ,
         * 'song' : 'SPICE UP YOUR LIFE' ,
         * 'channel' : '2' ,
         * 'amazonUrl' : 'http://www.amazon.com/gp/product/B000000WFF%3ftag=radiosherpa-20%26link_code=xm2%26camp=2025%26dev-t=065T8A7P2S5R0F5DNT82' 
       *}] 
       * }
       * 
       *
       */
    getStationInformation: function(stationlist,timespec,async) {
        var url=encodeURI(RadioSherpa.Config.site + "GetStationInformation?ResultFormat=json");
        var compFunc=function(transport,json) {
            var resptxt = transport.responseText;
            var called = false;
            if(transport.status == 200) {
		try{
		    var res=json;
		    if(undefined == json){
			res=eval('(' + resptxt + ')');
		    }
		    if(async!= null) async(res.results,res.time_stamp);
		    return {results:res.results,timestamp:res.time_stamp};
		}catch(e){
		    RadioSherpa.Config.exception(e);
		    RadioSherpa.Config.debug("Abnormal Response:\n" + resptxt + "");
		    if(async!=null) async([],undefined);
		    return [[],undefined];
		}
            }else{
		//               RadioSherpa.Config.exception(e);
		RadioSherpa.Config.debug("(GetStationInformation)Erroneous Response:\n" + dumpObj(transport) + "");
		if(async!=null) async([],undefined);
		return {results:[],timestamp:undefined};
            }

        };
        var since=0;
        var delta_ms = parseInt(timespec);
        delta_ms=isNaN(delta_ms)?0:delta_ms;
        if(timespec && timespec.since){
            since=parseFloat(timespec.since);
            since=isNaN(since)?0:since;
        }
        var req=RadioSherpa.http_post(url,async?compFunc:null,RadioSherpa.exceptionFunc);
        req.setRequestHeader("content-type","application/x-www-form-urlencoded");
        req.send(RadioSherpa.toQueryString($H({stations: stationlist.join(","),
					       delta_ms:delta_ms,
					       since:since,
					       Debug:"yes"})));
        url=null;
        if(async==null) return compFunc(req,null) 


    },
    /**
         * Variant of getStationInformation
         * that retrievs smaller amount of data.
         */
    getUpdateStationInformation: function(stationuidlist,timespec,async){
	var url=encodeURI(RadioSherpa.Config.site + "GetUpdateStationInformation?ResultFormat=json");
        var compFunc=function(transport,json) {
            var resptxt=transport.responseText;
            var called=false;
            if(transport.status == 200) {
		try{
                    var res=json;
                    if(undefined== json){
			res=eval('(' + resptxt + ')');
                    }
                    if(async!= null) async(res.results,res.time_stamp);
                    return {results:res.results,timestamp:res.time_stamp};
		}catch(e){
                    RadioSherpa.Config.exception(e);
                    RadioSherpa.Config.debug("Abnormal Response:\n" + resptxt + "");
                    if(async!=null) async([],undefined);
                    return {results:[],timestamp:undefined};
		}
            }else{
                //               RadioSherpa.Config.exception(e);
                RadioSherpa.Config.debug("(getUpdateStationInformation)Erroneous Response:\n" + dumpObj(transport) + resptxt + "");
                if(async!=null) async([],undefined);
                return {results:[],timestamp:undefined};
              }

            };
            var since=0;
            var delta_ms=parseInt(timespec);
            delta_ms=isNaN(delta_ms)?0:delta_ms;
            if(timespec && timespec.since){
              since=parseFloat(timespec.since);
              since=isNaN(since)?0:since;
            }
            var req=RadioSherpa.http_post(url,async?compFunc:null,RadioSherpa.exceptionFunc);
            req.setRequestHeader("content-type","application/x-www-form-urlencoded");
            req.send(RadioSherpa.toQueryString($H({stationuids:stationuidlist.join(","),
            delta_ms:delta_ms, since:since,    
            minor:1,
            Debug:"yes"}))); 
            url=null;
            if(async==null) return compFunc(req,null) 


          }
          ,
          /**
          * Get full station information based on criteria.
          * This method merges getStationList and getStationInformation into 
          * one call.
          * @param {hash/object} filter an object constructor or prototype Hash or JS Object containing the 
          * filtering criteria.
          * @param {function} async optional function to invoke with the results.
          * @returns a list of station information objects (@see getStationInformation)
          */
          getFullStationList: function(filter,async){
            var asyncfunc=function(lst) {
              var ids=[];
              lst.each(function(x){
                ids.push(x.id);
              });
              var result= RadioSherpa.getStationInformation(ids,0,async);
              return result;
            };
            var lst=RadioSherpa.getStationList(filter,async==null?null:asyncfunc) ;
            return asyncfunc(lst)
          }
        }
